/**
 * Common types, jQuery API, Form and Data controllers, utility functions.
 * @module data/common
 */
import type { Writable } from "svelte/store";
import type { JQueryApi } from "./semantic-types";
export declare const SVELTE_DATA_STORE: string;
export declare const SVELTE_FORM_STORE: string;
/** Return type for a simple svelte action; with destroy(), but without update(). */
export type ActionReturnType = {
    destroy: () => void;
} | void;
/** Format function, must return null if it can't parse value and doesn't want to override it. */
export interface Formatter {
    format: (val: DataTypes) => string;
    parse?: (val: string) => DataTypes;
}
export interface TextFormatter {
    format: (val: string) => string;
}
export interface NumberFormatter {
    format: (val: number | undefined) => string;
    parse: (val: string) => number | undefined;
}
export interface DateFormatter {
    format: (val: Date | undefined) => string;
    parse: (val: string) => Date | undefined;
}
export interface ListFormatter {
    format: (val: string[]) => string;
    parse: (val: string) => string[];
}
/** Validation rule object: rule string and custom error prompt */
export type RuleObj = {
    type: string;
    prompt?: string;
};
/** Rule definition takes array or single instance of string or RuleObj */
export type RuleDefinition = string | string[] | RuleObj | RuleObj[];
/** Gets jQuery element by id attribute. */
export declare function jQueryElemById(id: string): JQueryApi;
/** Gets jQuery element by dom node. */
export declare function jQueryElem(node: Element): JQueryApi;
export type ComponentInitMode = "parent" | "child" | "sibling";
/** How Init** components are looking for a Semantic UI input */
export declare function getComponentInitMode(): ComponentInitMode[];
/** Change component finding algorithm.
Setting to all modes will find component in any of the 3 locations.
Setting to empty array make <InitComp work only using forId. */
export declare function setComponentInitMode(modes: ComponentInitMode[]): void;
/** Find corresponding Semantic UI component,
where the `span` Element is a parent, child, or next sibling of the component.
Search mode is usually set in setComponentInitMode(), but may be overriden by `search` paremeter. */
export declare function findComponent(span: Element, selector: string, id?: string, search?: ComponentInitMode[]): JQueryApi;
/** Possible data types for the store */
export type DataTypes = string | string[] | boolean | Date | number | number[] | undefined;
/** Controls Semantic UI form element and it's data validation. */
export interface FormController {
    addRule: (key: string, rules: RuleDefinition) => void;
    removeRule: (key: string, rules: RuleDefinition) => void;
    doValidateField: (key: string) => void;
    doValidateForm: () => void;
    onFieldChange: (key: string) => void;
    validCallback?: (valid: boolean) => void;
    errorsCallback?: (errors: string[]) => void;
}
/** Holds Svelte `store` allowing the `Data` component to subscribe to data changes */
export interface DataController<T extends DataTypes> {
    /** Is used for debugging. */
    uid: string;
    /** Describes the type of the controlled component. */
    mode: "dropdown" | "modal" | "calendar" | "slider" | "input";
    /** Svelte store */
    store: Writable<T>;
    /** Imperatively updates Semantic component. */
    doUpdate: (val: T) => void;
    /** Called after user data entry or interaction. */
    onChange: (value: T) => void;
}
/** Semantic UI component behaviour API */
export type SemanticCommand = (command: string, v1?: unknown, v2?: unknown, v3?: unknown) => unknown;
/** Generate an unique number */
export declare function uid(): string;
/** Compare two arrays, two Date objects, or two primitives */
export declare function equalDataTypes(a1: DataTypes | undefined, a2: DataTypes | undefined): boolean;
export declare function pad(n: number, size: number): string;
export declare function isoDate(d: Date | undefined): string;
export declare function isoTime(d: Date | undefined): string;
