/**
 * Common types, jQuery API, Form and Data controllers, utility functions.
 * @module data/common
 */
export const SVELTE_DATA_STORE = "svelte_data_store";
export const SVELTE_FORM_STORE = "svelte_form_store";
/** Gets jQuery element by id attribute. */
export function jQueryElemById(id) {
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized");
    }
    return jQuery(`#${id}`);
}
/** Gets jQuery element by dom node. */
export function jQueryElem(node) {
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized");
    }
    return jQuery(node);
}
const ALL_MODES = ["parent", "child", "sibling"];
/** Global setting for finding corresponding Semantic UI component */
let COMPONENT_INIT_MODES = ["sibling"];
/** How Init** components are looking for a Semantic UI input */
export function getComponentInitMode() {
    return COMPONENT_INIT_MODES;
}
/** Change component finding algorithm.
Setting to all modes will find component in any of the 3 locations.
Setting to empty array make <InitComp work only using forId. */
export function setComponentInitMode(modes) {
    modes.forEach((mode) => {
        if (!ALL_MODES.includes(mode)) {
            throw new Error(`Invalid component init mode: ${mode}`);
        }
    });
    COMPONENT_INIT_MODES = modes;
}
/** Find corresponding Semantic UI component,
where the `span` Element is a parent, child, or next sibling of the component.
Search mode is usually set in setComponentInitMode(), but may be overriden by `search` paremeter. */
export function findComponent(span, selector, id, search) {
    let elem;
    if (id) {
        // use "forId" attribute to find the element to connect to
        elem = jQueryElemById(id);
        if (elem.length && elem.is(selector)) {
            return elem;
        }
        throw new Error(`Can't find '${selector}' component with id="${id}"`);
    }
    const searchModes = search ? search : COMPONENT_INIT_MODES;
    if (searchModes.includes("parent")) {
        elem = jQueryElem(span).find(selector);
        if (elem.length > 0) {
            return elem;
        }
    }
    if (searchModes.includes("child")) {
        elem = jQueryElem(span).parent(selector);
        if (elem.length > 0) {
            return elem;
        }
    }
    if (searchModes.includes("sibling")) {
        elem = jQueryElem(span).prev(selector);
        if (elem.length > 0) {
            return elem;
        }
    }
    throw new Error(`Can't find '${selector}' component as a ${searchModes.join("|")}`);
}
/*
            dP   oo dP
            88      88
 dP    dP d8888P dP 88
 88    88   88   88 88
 88.  .88   88   88 88
 `88888P'   dP   dP dP

*/
let unum = 100;
/** Generate an unique number */
export function uid() {
    // const num = new Date().getTime();
    // const num = Math.round(window.performance.now());
    unum = unum + 1;
    return `000000${unum % 1000}`.slice(-3);
}
/** Compare two arrays, two Date objects, or two primitives */
export function equalDataTypes(a1, a2) {
    if (Array.isArray(a1) && Array.isArray(a2)) {
        if (a1.length !== a2.length) {
            return false;
        }
        for (let i = 0; i < a1.length; i++) {
            if (a1[i] !== a2[i]) {
                return false;
            }
        }
        return true;
    }
    if (a1 instanceof Date && a2 instanceof Date) {
        return a1.getTime() === a2.getTime();
    }
    return a1 === a2;
}
export function pad(n, size) {
    let str = n.toString();
    while (str.length < size) {
        str = `0${str}`;
    }
    return str;
}
export function isoDate(d) {
    if (!d || !d.getDate) {
        return "";
    }
    const day = pad(d.getDate(), 2);
    const month = pad(d.getMonth() + 1, 2);
    const year = pad(d.getFullYear(), 4);
    return `${year}-${month}-${day}`;
}
export function isoTime(d) {
    if (!d || !d.getDate) {
        return "";
    }
    const hour = pad(d.getHours(), 2);
    const minute = pad(d.getMinutes(), 2);
    return `${hour}:${minute}`;
}
