/**
 * 'fmt', 'parse', and 'rile' utility object.
 * @module data/helpers
 */
import { DateFmt } from "./input-formatter";
/*
 .8888b              dP
 88   "              88
 88aaa  88d8b.d8b. d8888P
 88     88'`88'`88   88
 88     88  88  88   88
 dP     dP  dP  dP   dP

*/
// Global fmt and parse objects with default formatting settings.
// Lazily initialized, to give user the chance to change defaults.
let defaultDateFmt = null;
function getDefaultDateFmt() {
    if (defaultDateFmt === null) {
        defaultDateFmt = new DateFmt({ type: "date" });
    }
    return defaultDateFmt;
}
let defaultTimeFmt = null;
function getDefaultTimeFmt() {
    if (defaultTimeFmt === null) {
        defaultTimeFmt = new DateFmt({ type: "time" });
    }
    return defaultTimeFmt;
}
export const fmt = {
    date: (d) => getDefaultDateFmt().format(d),
    time: (d) => getDefaultTimeFmt().format(d),
};
/*

 88d888b. .d8888b. 88d888b. .d8888b. .d8888b.
 88'  `88 88'  `88 88'  `88 Y8ooooo. 88ooood8
 88.  .88 88.  .88 88             88 88.  ...
 88Y888P' `88888P8 dP       `88888P' `88888P'
 88
 dP
*/
// TODO: review null/undefined, decide which one should be used
export const parse = {
    date: (value) => {
        if (!value) {
            return undefined;
        }
        return getDefaultDateFmt().parse(value);
    },
    time: (value) => {
        if (!value) {
            return undefined;
        }
        return getDefaultTimeFmt().parse(value);
    },
};
/*
                   dP             dP                dP
                   88             88                88
 88d888b. dP    dP 88 .d8888b.    88d888b. .d8888b. 88 88d888b. .d8888b. 88d888b.
 88'  `88 88    88 88 88ooood8    88'  `88 88ooood8 88 88'  `88 88ooood8 88'  `88
 88       88.  .88 88 88.  ...    88    88 88.  ... 88 88.  .88 88.  ... 88
 dP       `88888P' dP `88888P'    dP    dP `88888P' dP 88Y888P' `88888P' dP
                                                       88
                                                       dP
*/
/** Autocomplete helper for validation rules */
/* prettier-ignore */
// eslint-disable-next-line @typescript-eslint/typedef
export const rule = {
    /* eslint-disable key-spacing */
    empty: () => "empty",
    checked: () => "checked",
    email: () => "email",
    url: () => "url",
    integer: () => "integer",
    decimal: () => "decimal",
    number: () => "number",
    creditCard: () => "creditCard",
    regex: (reg) => `regExp[//${reg}//]`,
    // integer:    (min?: number, max?: number): string =>
    //     min == undefined && max == undefined ? "integer" : `integer[${min}..${max}]`,
    is: (val) => `is[${val}]`,
    isExactly: (val) => `isExactly[${val}]`,
    not: (val) => `not[${val}]`,
    notExactly: (val) => `notExactly[${val}]`,
    contains: (val) => `contains[${val}]`,
    containsExactly: (val) => `containsExactly[${val}]`,
    doesntContain: (val) => `doesntContain[${val}]`,
    doesntContainExactly: (val) => `doesntContainExactly[${val}]`,
    match: (fld) => `match[${fld}]`,
    different: (fld) => `different[${fld}]`,
    minLength: (n) => `minLength[${n}]`,
    exactLength: (n) => `exactLength[${n}]`,
    maxLength: (n) => `maxLength[${n}]`,
    minCount: (n) => `minCount[${n}]`,
    exactCount: (n) => `exactCount[${n}]`,
    maxCount: (n) => `maxCount[${n}]`,
    // helpers for custom rules
    start: (val) => `start[${val}]`,
    isoDate: () => "isoDate",
    startEnd: (val) => `startEnd[${val}]`,
    // TODO: investigate these rules
    range: "",
    maxValue: "",
    minValue: "",
    size: "",
    /* eslint-enable */
};
/*
          dP                                             dP
          88                                             88
 .d8888b. 88d888b. .d8888b. 88d888b. .d8888b. .d8888b. d8888P
 88'  `"" 88'  `88 88'  `88 88'  `88 Y8ooooo. 88ooood8   88
 88.  ... 88    88 88.  .88 88             88 88.  ...   88
 `88888P' dP    dP `88888P8 dP       `88888P' `88888P'   dP

*/
// type Range = [number, number];
// type Charset = Array<Range>;
// function ch(s: string): number {
//     return s.codePointAt(0) ?? 0;
// }
// function chRng(s: string): Range {
//     return [ch(s), ch(s)];
// }
// export const NUMER: Charset = [[ch("0"), ch("9")]];
// export const ALPHA: Charset = [
//     [ch("A"), ch("Z")],
//     [ch("a"), ch("z")],
// ];
// export const ALPNU: Charset = [...NUMER, ...ALPHA];
// export const IDENT: Charset = [...ALPNU, chRng("_")];
// export const ASCII: Charset = [[ch(" "), ch("~")]];
// void new CharsetList({ base: IDENTIFIER, inclide: "-.", exclude: "_" });
