/**
 * 'fmt' and 'parse' utility object, input formatter classes.
 * @module data/input-formatter
 */
import { calendarDefaults, numberDefaults } from "../data/settings";
/*
                              dP
                              88
 88d888b. dP    dP 88d8b.d8b. 88d888b. .d8888b. 88d888b.
 88'  `88 88    88 88'`88'`88 88'  `88 88ooood8 88'  `88
 88    88 88.  .88 88  88  88 88.  .88 88.  ... 88
 dP    dP `88888P' dP  dP  dP 88Y8888' `88888P' dP

*/
/** Encode . * + ? ^ $ { } ( ) | [ ] \ to do litral match in regex */
function escapeRegExp(val) {
    return val.replace(/[.*+?^${}()|[\]\\]/g, "\\$&").trim();
}
export class NumberFmt {
    constructor(inputSettings) {
        if (inputSettings.type && !["integer", "decimal", "money"].includes(inputSettings.type)) {
            throw new Error(`Unsupported number type: ${inputSettings.type}`);
        }
        if (inputSettings.precision && Math.abs(inputSettings.precision) > 6) {
            throw new Error(`Unsupported precision number ${inputSettings.precision}`);
        }
        const def = numberDefaults.read();
        this.settings = {
            ...def,
            ...inputSettings,
        };
        this.type = inputSettings.type ?? "integer";
        this.precision =
            inputSettings.precision ?? (this.type === "money" ? this.settings.moneyPrecision : 0);
        if (this.type === "money") {
            this.prefixRegex = new RegExp(`^\\s*${escapeRegExp(this.settings.moneyPrefix)}\\s*`);
            this.suffixRegex = new RegExp(`\\s*${escapeRegExp(this.settings.moneySuffix)}\\s*$`);
        }
    }
    // TODO: move format / parse number into format.ts
    parse(value) {
        let val = value;
        if (this.type === "money") {
            val = val.replace(this.prefixRegex, "").replace(this.suffixRegex, "");
        }
        val = val.replace(/\s/g, ""); // remove whitespace
        val = val.split(this.settings.thousandSeparator).join(""); // replace all
        val = val.replace(this.settings.decimalSeparator, ".");
        const num = parseFloat(val);
        if (Number.isNaN(num)) {
            return undefined;
        }
        const pwr = 10.0 ** -this.precision;
        // console.log("PWR", pwr, val, num, Math.round(num / pwr) * pwr);
        const res = Math.round(num / pwr) * pwr;
        return Number(res.toFixed(6));
    }
    format(val) {
        if (val === undefined) {
            return "";
        }
        if (typeof val !== "number") {
            throw new Error(`numberFormatter expects number as data type, got ${typeof val}`);
        }
        let str = val.toFixed(Math.max(this.precision, 0));
        str = str.replace(".", this.settings.decimalSeparator);
        const len = str.length;
        const firstPos = this.precision > 0 ? this.precision + 1 : 0;
        for (let n = 3 + firstPos; n < len; n += 3) {
            str =
                str.substring(0, len - n) +
                    this.settings.thousandSeparator +
                    str.substring(len - n);
        }
        if (this.type === "money") {
            str = this.settings.moneyPrefix + str + this.settings.moneySuffix;
        }
        return str;
    }
}
export class TextFmt {
    constructor(settings) {
        this.caseMode = settings?.case ?? "none";
    }
    format(val) {
        if (typeof val !== "string") {
            throw new Error("textFormatter expects string as data type");
        }
        switch (this.caseMode) {
            case "none":
                return val;
            case "upper":
                return val.trim().toUpperCase();
            case "lower":
                return val.trim().toLowerCase();
            case "title":
                return val
                    .trim()
                    .split(" ")
                    .map((s) => s.charAt(0).toUpperCase() + s.substring(1).toLowerCase())
                    .join(" ");
            default:
                throw new Error(`Unrecognized case ${this.caseMode}`);
        }
    }
}
/*
 dP oo            dP
 88               88
 88 dP .d8888b. d8888P
 88 88 Y8ooooo.   88
 88 88       88   88
 dP dP `88888P'   dP

*/
export class ListFmt {
    constructor(sep) {
        if (sep === "") {
            throw new Error("List separator can't be empty");
        }
        this.separator = sep !== undefined ? sep : numberDefaults.read().listSeparator;
    }
    parse(val) {
        if (!val) {
            return [];
        }
        let sep = this.separator.trim();
        if (sep === "") {
            sep = " ";
        }
        return val.split(this.separator).map((item) => item.trim());
    }
    format(val) {
        if (!Array.isArray(val)) {
            throw new Error(`listFormatter expects string[] as data type, got ${typeof val}`);
        }
        return val.join(this.separator);
    }
}
/*
       dP            dP
       88            88
 .d888b88 .d8888b. d8888P .d8888b.
 88'  `88 88'  `88   88   88ooood8
 88.  .88 88.  .88   88   88.  ...
 `88888P8 `88888P8   dP   `88888P'

 */
export class DateFmt {
    constructor(settings) {
        this.settings = {
            ...calendarDefaults.read(),
            ...settings,
            // TODO: do i need to use deep copy?
        };
    }
    /* eslint-disable one-var */
    /* eslint-disable prefer-template */
    /* eslint-disable no-nested-ternary */
    /* eslint-disable @typescript-eslint/typedef */
    // this function is copied from fomantic-ui calendar component v2.9.3
    weekOfYear(weekYear, weekMonth, weekDay) {
        void this;
        // adapted from http://www.merlyn.demon.co.uk/weekcalc.htm
        const ms1d = 24 * 3600 * 1000, ms7d = 7 * ms1d, DC3 = Date.UTC(weekYear, weekMonth, weekDay + 3) / ms1d, // an absolute day number
        AWN = Math.floor(DC3 / 7), // an absolute week number
        WYR = new Date(AWN * ms7d).getUTCFullYear();
        return AWN - Math.floor(Date.UTC(WYR, 0, 7) / ms7d) + 1;
    }
    // this function is copied from fomantic-ui calendar component v2.9.3
    helperDateFormat(format, date, sett) {
        const settings = sett ?? calendarDefaults.read();
        if (!(date instanceof Date)) {
            return "";
        }
        if (typeof format === "function") {
            return format.call(this, date, settings);
            // TODO: sest if it works with formatter function
        }
        const D = date.getDate(), M = date.getMonth(), Y = date.getFullYear(), d = date.getDay(), H = date.getHours(), m = date.getMinutes(), s = date.getSeconds(), w = /* module.get. */ this.weekOfYear(Y, M, D + 1 - settings.firstDayOfWeek), h = H % 12 || 12, a = H < 12 ? settings.text.am.toLowerCase() : settings.text.pm.toLowerCase(), tokens = {
            D: D,
            DD: ("0" + D).slice(-2),
            M: M + 1,
            MM: ("0" + (M + 1)).slice(-2),
            MMM: settings.text.monthsShort[M],
            MMMM: settings.text.months[M],
            Y: Y,
            YY: String(Y).slice(2),
            YYYY: Y,
            d: d,
            dd: settings.text.days[d],
            ddd: settings.text.days[d],
            dddd: settings.text.days[d],
            // dd: settings.text.dayNamesShort[d].slice(0, 2),
            // ddd: settings.text.dayNamesShort[d],
            // dddd: settings.text.dayNames[d],
            h: h,
            hh: ("0" + h).slice(-2),
            H: H,
            HH: ("0" + H).slice(-2),
            m: m,
            mm: ("0" + m).slice(-2),
            s: s,
            ss: ("0" + s).slice(-2),
            a: a,
            A: a.toUpperCase(),
            S: ["th", "st", "nd", "rd"][D % 10 > 3 ? 0 : (D % 100) - (D % 10) === 10 ? 0 : D % 10],
            w: w,
            ww: ("0" + w).slice(-2),
        };
        // eslint-disable-next-line func-names, prefer-arrow-callback
        return format.replace(settings.regExp.token, function (match) {
            if (match in tokens) {
                return tokens[match];
            }
            return match.slice(1, -1);
        });
    }
    /* eslint-enable */
    parse(val) {
        if (!val) {
            return undefined;
        }
        const dateParser = this.settings.parser.date;
        return dateParser(val, this.settings) ?? undefined;
    }
    format(val) {
        if (val === undefined || val === null) {
            return "";
        }
        if (!(val instanceof Date)) {
            throw new Error(`dateFormatter expects Date as data type, got ${typeof val} = ${val}`);
        }
        const type = this.settings.type;
        return this.helperDateFormat(this.settings.formatter[type], val, this.settings);
    }
}
