/**
 * Semantic UI types and helper to access settings.
 *
 * Re-exports SemanitUI types from the module instead of namespaces.
 *
 * Fixes type definition inconsistencies.
 * @module data/semantic-ui-types
 */
/// <reference types="jquery" />
/// <reference types="fomantic-ui-css" />
/* <reference> lines above were added manually to fix Svelte 5 packaging bug
 */
import type { RuleDefinition } from "./common";
/** jQuery */
export type JQueryApi = JQuery<HTMLElement>;
export type DateFormatFn = (d: Date | undefined, settings?: CalendarSettings) => string;
export type DateParseFn = (date: string | undefined, settings?: CalendarSettings) => Date | undefined;
/**
 * Fomantic UI Calendar settings.
 * Adds 'formatter' documetation and the fix for 'onChange' arguments.
 *
 * @see {@link https://fomantic-ui.com/modules/calendar.html#/settings}
 */
export type CalendarSettings = Omit<Partial<FomanticUI.CalendarSettings>, "onChange"> & {
    /** Is called after a calendar date has changed. */
    onChange?(this: JQuery<HTMLElement>, date: Date, text: string, mode: string): void;
    formatter?: {
        cellTime?: string | DateFormatFn;
        date?: string | DateFormatFn;
        datetime?: string | DateFormatFn;
        time?: string | DateFormatFn;
        month?: string | DateFormatFn;
        year?: string | DateFormatFn;
        dayHeader?: string | DateFormatFn;
        hourHeader?: string | DateFormatFn;
        minuteHeader?: string | DateFormatFn;
        monthHeader?: string | DateFormatFn;
        yearHeader?: (date: Date, settings: CalendarSettings) => string;
        cell?: (cell: JQuery, date: Date, options: {
            mode: string;
            adjacent: boolean;
            disabled: boolean;
            active: boolean;
            today: boolean;
        }) => string;
        dayColumnHeader?: (day: number, settings: CalendarSettings) => string;
        today?: (settings: CalendarSettings) => string;
    };
    parser?: {
        date?: DateParseFn;
    };
};
export type CalendarText = FomanticUI.Calendar.Settings.Texts;
export type RuleFunc = (value: string, ruleValue: string, form: JQuery) => boolean;
export type FormRules = {
    [key: string]: RuleFunc;
};
/**
 * Fomantic UI form validation behavior.
 *
 * @see {@link https://fomantic-ui.com/behaviors/form.html#/settings}
 */
export type FormSettings = Partial<FomanticUI.FormSettings> & {
    fields?: Record<string, RuleDefinition>;
    rules?: FormRules;
};
export type FormPropmt = Omit<FomanticUI.Form.Settings.Prompts, "contain" | "containExactly"> & {
    contains: string;
    containsExactly: string;
    start: string;
    isoDate: string;
    startEnd: string;
};
export type FormText = FomanticUI.Form.Settings.Texts;
/** Settings contriolling number, money, and list formatting. */
export interface NumberSettings {
    decimalSeparator: string;
    thousandSeparator: string;
    moneyPrefix: string;
    moneySuffix: string;
    moneyPrecision: number;
    listSeparator: string;
}
/** Settings used to initailaze a number input component. */
export interface NumberInputSettings extends Partial<NumberSettings> {
    type?: "integer" | "decimal" | "money";
    precision?: number;
}
/** @see {@link http://fomantic-ui.com/modules/dropdown.html#/settings} */
export type DropdownSettings = Omit<Partial<FomanticUI.DropdownSettings>, "onChange"> & {
    /**
     * Is called after a dropdown value changes.
     * Receives the name and value of selection and the active menu element.
     */
    onChange?(value: string | string[], text: string, $choice: JQuery): void;
};
export type DropdownMessages = FomanticUI.Dropdown.Settings.Messages;
/** @see {@link https://fomantic-ui.com/modules/progress.html#/settings} */
export type ProgressSettings = Partial<FomanticUI.ProgressSettings>;
/**
 * Text content for each state, uses simple templating with {percent}, {value}, {total} and {bar}.

   {bar} is useful to show bar names on multiple bars.
   Names of bars are provided by text.bars in the forms of array of string. E.g. ['bar1', 'bar2'].
 */
export type ProgressTexts = Partial<FomanticUI.Progress.Settings.Texts>;
export type FlyoutTexts = {
    ok: string;
    cancel: string;
    close: string;
};
/** @see {@link https://fomantic-ui.com/modules/flyout.html#/settings} */
export type FlyoutSettings = Partial<FomanticUI.FlyoutSettings> & {
    text: FlyoutTexts;
};
/** @see {@link https://fomantic-ui.com/modules/checkbox.html#/settings} */
export type CheckboxSettings = Partial<FomanticUI.CheckboxSettings>;
/** @see {@link https://fomantic-ui.com/modules/modal.html#/settings} */
export type ModalSettings = Partial<FomanticUI.ModalSettings>;
/** @see {@link https://fomantic-ui.com/modules/popup.html#/settings} */
export type PopupSettings = Partial<FomanticUI.PopupSettings>;
/** @see {@link https://fomantic-ui.com/modules/slider.html#/settings} */
export type SliderSettings = Partial<FomanticUI.SliderSettings>;
/** @see {@link https://fomantic-ui.com/modules/sticky.html#/settings} */
export type StickySettings = Partial<FomanticUI.StickySettings>;
/** @see {@link https://fomantic-ui.com/modules/toast.html#/settings} */
export type ToastSettings = Partial<FomanticUI.ToastSettings>;
