/**
 * Common types, jQuery API, Form and Data controllers, utility functions.
 * @module data/settings
 */
/** Determine if the argument is object; not primitive, Array, of function */
function isObject(value) {
    return value !== null && typeof value === "object" && !Array.isArray(value);
}
/** Recursive function to copy only matching fields. */
function copyFields(target, source, logName) {
    const sourceKeys = Object.keys(source);
    const targetKeys = Object.keys(target);
    if (sourceKeys.length === 0 || targetKeys.length === 0) {
        return;
    }
    sourceKeys.forEach((key) => {
        if (targetKeys.includes(key)) {
            const to = isObject(target[key]);
            const so = isObject(source[key]);
            if (to) {
                if (so) {
                    // obj <- obj
                    copyFields(target[key], source[key], `${logName}.${key}`);
                }
                else {
                    // obj <- prim|fn|arr
                    console.log(`Setting: '${key}' in ${logName} must be an Object.`);
                }
            }
            else {
                if (so) {
                    // prim|fn|arr <- obj
                    console.log(`Setting: '${key}' in ${logName} can't be an Object.`);
                }
                else {
                    // prim|fn|arr <- prim|fn|arr
                    target[key] = source[key];
                }
            }
        }
        else {
            console.log(`Unrecognized setting: '${key}' in ${logName}.`);
        }
    });
}
// /** Utility type to mark all fields and sub-objects Readonly. */
// export type DeepReadonly<T> = T extends unknown
//     ? { readonly [K in keyof T]: DeepReadonly<T[K]> }
//     : T;
function ensureNumberSettings(jQuery) {
    if (!jQuery.fn.number) {
        jQuery.fn.number = {
            settings: {
                decimalSeparator: ".", // TODO: implement onfigurable separator
                thousandSeparator: " ",
                moneyPrefix: "$",
                moneySuffix: "",
                moneyPrecision: 2,
                listSeparator: ", ",
            },
        };
    }
}
/*
 dP                dP
 88                88
 88d888b. .d8888b. 88 88d888b. .d8888b. 88d888b.
 88'  `88 88ooood8 88 88'  `88 88ooood8 88'  `88
 88    88 88.  ... 88 88.  .88 88.  ... 88
 dP    dP `88888P' dP 88Y888P' `88888P' dP
                      88
                      dP
*/
/** Provides acces to Sematic UI settings for different components. */
export class SettingsHelper {
    constructor(compName) {
        this.compName = compName;
    }
    /** Returns settings object. */
    read() {
        const jQuery = window.jQuery;
        if (!jQuery) {
            throw new Error("jQuery is not initialized.");
        }
        ensureNumberSettings(jQuery);
        const settings = jQuery.fn[this.compName]?.settings;
        if (!settings) {
            throw new Error(`Semantic UI ${this.compName} is not initialized.`);
        }
        return settings;
    }
    /** Copy settings into the global SUI components in jQuery. */
    apply(val) {
        const jQuery = window.jQuery;
        if (!jQuery) {
            throw new Error("jQuery is not initialized.");
        }
        ensureNumberSettings(jQuery);
        const settings = jQuery.fn[this.compName]?.settings;
        if (!settings) {
            throw new Error(`Semantic UI ${this.compName} is not initialized.`);
        }
        copyFields(settings, val, this.compName);
    }
}
/*
                            dP
                            88
 .d8888b. 88d888b. 88d888b. 88 dP    dP
 88'  `88 88'  `88 88'  `88 88 88    88
 88.  .88 88.  .88 88.  .88 88 88.  .88
 `88888P8 88Y888P' 88Y888P' dP `8888P88
          88       88               .88
          dP       dP           d8888P
*/
/** Copy settings into the global SUI components and number object in jQuery. */
export function applyAllSettings(json) {
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized.");
    }
    ensureNumberSettings(jQuery);
    if (!json) {
        return;
    }
    // eslint-disable-next-line no-restricted-syntax, guard-for-in
    Object.keys(json).forEach((name) => {
        const settings = jQuery.fn[name]?.settings;
        if (!settings) {
            console.log(`Ignoring unrecognized Semantic UI component: ${name}`);
        }
        else {
            copyFields(settings, json[name], name);
        }
    });
}
/*
 oo          oo   dP   oo          dP oo
                  88               88
 dP 88d888b. dP d8888P dP .d8888b. 88 dP d888888b .d8888b.
 88 88'  `88 88   88   88 88'  `88 88 88    .d8P' 88ooood8
 88 88    88 88   88   88 88.  .88 88 88  .Y8P    88.  ...
 dP dP    dP dP   dP   dP `88888P8 dP dP d888888P `88888P'

*/
export const calendarDefaults = new SettingsHelper("calendar");
export const checkboxDefaults = new SettingsHelper("checkbox");
export const dropdownDefaults = new SettingsHelper("dropdown");
export const formDefaults = new SettingsHelper("form");
export const modalDefaults = new SettingsHelper("modal");
export const numberDefaults = new SettingsHelper("number");
export const popupDefaults = new SettingsHelper("popup");
export const sliderDefaults = new SettingsHelper("slider");
export const stickyDefaults = new SettingsHelper("sticky");
export const toastDefaults = new SettingsHelper("toast");
/** Must be called after DOM is initailized. Like in sveltekit rotes/layout.ts load(). */
export function applyDefaultSettings(locale) {
    void locale;
    calendarDefaults.apply({
        type: "date",
        touchReadonly: false,
        minTimeGap: 5,
    });
    formDefaults.apply({
        keyboardShortcuts: false,
    });
}
